#ifndef __INC_HTTPREADER_H__
#define __INC_HTTPREADER_H__

#include "kurl.h"
#include "qobject.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <netdb.h>
#include <stdio.h>
#include <qdatetm.h>
#include <qlist.h>
#include <klocale.h>

#define KIO_ERROR_CouldNotWrite 1
#define KIO_ERROR_CouldNotRead 2
#define KIO_ERROR_CouldNotDelete 3
#define KIO_ERROR_CouldNotCreateSocket 4
#define KIO_ERROR_UnknownHost 5
#define KIO_ERROR_CouldNotConnect 7
#define KIO_ERROR_CouldNotMount 8
#define KIO_ERROR_CouldNotUnmount 9
#define KIO_ERROR_NotImplemented 10
#define KIO_ERROR_CouldNotLogin 11
#define KIO_ERROR_MalformedURL 12
#define KIO_ERROR_CouldNotMkdir 13
#define KIO_ERROR_CouldNotList 14
#define KIO_ERROR_TarError 15
#define KIO_ERROR_GzipError 16
#define KIO_ERROR_FileExists 17

/**
 * Use this if it is technically impossible to fullfill the
 * request. For example it is impossible to list the directory
 * structure of a GZIP file.
 */

#define KIO_ERROR_NotPossible 18
#define KIO_ERROR_SlaveDied 19
#define KIO_ERROR_FileDoesNotExist 20
#define KIO_ERROR_NotADirectory 21
#define KIO_ERROR_DiskFull 22

enum Status { FAIL=-1, SUCCESS=0};
enum Kind { READ=1, WRITE=2, OVERWRITE=4 };

void Error(int KError, QString ErrorMessage, int SysError);

class CHTTPReader : public QObject
{
	Q_OBJECT

protected:
	
	QList<char> *agent_strings;
	QString noProxyForStr;    
	int use_proxy;
	QString proxy_user;
	QString proxy_pass;
	QString post_data;
	QString Cookies;

public:
	
	QString url;
	QString lastErrorMessage;
	int lastKError;
	int lastSysError;
  bool bIntranet;
signals:
	void info (const char *);
	void redirection(const char *);
	void mimeType(const char *);
	void cookie(const char *, const char *);
protected:    
// needed for language/charset support 
//    KLocale *klocale;

    //#define HTTP_PROXY_NAME "someproxy.somewhere.org"
    //#define HTTP_PROXY_PORT 80
    
	struct sockaddr_in proxy_name;
	int connected, sock;
	long size;
	long bytesleft;
	long bytesRead;
	FILE *fsocket;

	// Jacek:
	// HTTP settings
	bool assumeHTML;
	QString languages;
	QString charsets;

  /**
   * Used to store the parameter given by a call to @ref #Open. We need this
   * for HTTP Redirection.
   *
   * @see #ProcessHeader
   * @see #Open
   */
  int currentMode;

	QTime currentTime;
	QTime startTime;
  
	void PrepareLanguageList(QString str);
	void PrepareCharsetList(QString str);
	int init_sockaddr(struct sockaddr_in *server_name, const char *hostname,int port);
	int ProcessHeader();
	int OpenHTTP(KURL *url, int mode,bool reload);
  
public:
	CHTTPReader();
	~CHTTPReader();
  
	virtual int Open(KURL *url, int mode);
	virtual int ReOpen(KURL *url, int mode);
	virtual int Close();
	virtual int SetData(const char *_data);
	virtual int SetCookies(const char *_cookies);
	virtual long Read(void *buffer, long len);
	virtual long Size();
	virtual int atEOF();
public slots:
  void OnRedirection(const char *);
};

extern "C"
{
	int revmatch (const char *host, const char *nplist);
	char *base64_encode_line (const char *s);
	char *create_generic_auth (const char *prefix, const char *user, const char *papasswd);
	char *create_www_auth (const char *user, const char *passwd);
};

#endif /* __INC_HTTPREADER_H__ */

