/**
 * @author Torben Weis, weis@kde.org
 * @short Executes random URLs
 *
 *
 * Modifications to this program were made by Corel Corporation,
 * November, 1999.  All such modifications are copyright (C) 1999
 * Corel Corporation and are licensed under the terms of the LGPL License.
			 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.

 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#ifndef _kfmexec_h
#define _kfmexec_h

#include <qstring.h>

class KURL;
class KMimeType;
class OpenWithDlg;

/* 
 * If you have a single URL and you want to run the default action, then just
 * create a KFMExec instance and call @ref #openURL.
 * The class tries to determine wether the URL is a directory or a HTML file
 * or something different. For directories and HTML a new KFM window
 * will be opened. For all the other files we try to find out the mime type
 * and run the default binding. If we have no success, the user is asked
 * what to do. There is no need for you to destroy KFMExec jobs. They
 * kill themselves after they finished their job. So dont use the pointer
 * to a KFMExec class after calling @ref #openURL. KFMExec shows a window
 * if it talks to the Internet to show the user that something is done.
 * For example @ref KRootWidget uses this class to execute URL links
 * lying around on the desktop.
 */
class KFMExec 
{
public:
    /**
     * Tells KFMExec to run the default action for the given URL.
     * After calling this function dont reference this instance any
     * more. It will destroy itself after its job is done.
     */
    static void openURL( const char *_url, WId parentId = 0);
    
    /**
     * This function handles some special stuff for local files. For example
     * it changes the URL, if the file is zipped or a tar file. The changed
     * URL is then returned. Sometimes the function can execute the URL
     * itself ( executables, mime-type with binding ). Then an empty string
     * is returned. This function is used by KFMExec and @ref KFMManager.
     */
    static QString openLocalURL( const char *_filename, WId parentId = 0);
    static QString openLocalURL( KURL & u, WId parentId = 0);

		static bool showOpenWith(const char *tryURL, KMimeType *mime = 0, WId parent = 0);
};

#endif





