//------------------------------------------------------------------------------
// Name : smbconfig.h 
// 
// Description: Defines the main interface class of libccsmb
// 
// Copyright (C) 1999 Corel Corporation 
// 
// This program/binary is licensed under the LGPL license. (See 
// http://www.gnu.org/copyleft/lgpl.html) for details. 
//------------------------------------------------------------------------------

#ifndef __CSambaConfig__
#define __CSambaConfig__

#include <qdict.h>
#include <sys/types.h>

#include "smbobjtypes.h"

class CSmbSection;	// forward class declaration
class CSmbSectionList;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define	GLOBAL_SECTION_HEADER	("global")

// Global parameters
#define	DESCRIPTION_KEY			("server string")
#define	WORKGROUP_KEY				("workgroup")
#define	SECURITY_LEVEL_KEY	("security")
#define	OS_LEVEL_KEY				("os level")
#define DOMAIN_MASTER_KEY		("domain master")
#define LOCAL_MASTER_KEY		("local master")
#define PREFERED_MASTER_KEY	("prefered master")
#define DOMAIN_LOGONS_KEY		("domain logons")
#define	PASSWORD_SERVER_KEY	("password server")
#define	WINS_SUPPORT_KEY		("wins support")
#define	WINS_SERVER_KEY			("wins server")
#define	LOAD_PRINTERS_KEY		("load printers")

#define	CASE_SENSITIVE_KEY			("case sensitive")
#define	SHORT_PRESERVE_CASE_KEY	("short preserve case")
#define	PRESERVE_CASE_KEY				("preserve case")

// share level parameters
#define COMMENT_KEY					("comment")
#define PATH_KEY						("path")
#define LOCKING_KEY					("locking")
#define WRITEABLE_KEY				("writeable")
#define PUBLIC_KEY					("public")

// PDC parameters relating to the special [netlogon] share
#define PDC_NETLOGON_SHARE_NAME				("netlogon")

#define PDC_NETLOGON_COMMENT_VALUE		("NETLOGON service")
#define PDC_NETLOGON_PATH_VALUE				("/export/smb/netlogon")
#define PDC_NETLOGON_LOCKING_VALUE		(INI_NO)
#define PDC_NETLOGON_WRITEABLE_VALUE	(INI_NO)
#define PDC_NETLOGON_PUBLIC_VALUE			(INI_NO)

#define	DEFAULT_CASE_SENSITIVE_VALUE				(INI_NO)
#define	DEFAULT_SHORT_PRESERVE_CASE_VALUE		(INI_YES)
#define	DEFAULT_PRESERVE_CASE_VALUE					(INI_YES)

// For the os level parameter:
//		NT uses a value of 32,
//		NT workstation uses a value of 16
//		and Windows 95 and WfW use a value of 1
#define	DEFAULT_OS_LEVEL_VALUE							("0")
#define REGULAR_SERVER_OS_LEVEL_VALUE				("31")
#define PDC_SERVER_OS_LEVEL_VALUE						("31")

#define	SECURITY_LEVEL_SERVER	("SERVER")
#define	SECURITY_LEVEL_USER		("USER")
#define	SECURITY_LEVEL_SHARE	("SHARE")

#define	INI_YES		("yes")
#define	INI_NO		("no")

#define	DESCRIPTION_DEFAULT_STRING				("Local Machine Description")
#define	WORKGROUP_DEFAULT_STRING	 		 		("Windows Work Group or NT Domain name")
#define	DOMAIN_CONTROLLER_DEFAULT_STRING	("NT Domain  server")

#define	DEBIAN_SMB_SERVICE_NAME				("samba")
#define	DEBIAN_RUN_LEVEL_PARENT_PATH	("/etc/")

#define	REDHAT_SMB_SERVICE_NAME				("smb")
#define	REDHAT_RUN_LEVEL_PARENT_PATH	("/etc/rc.d/")

class CSambaConfig
{
// type definitions
public:
    typedef  enum { eServer = 0, eUser, eShare } SECURITY_LEVEL_T;
public:

    CSambaConfig(QWidget* parent = NULL, const char* name = NULL);

    virtual ~CSambaConfig();


		// save all temporary key/value pairs that are currently stored
		// returns a bool: if the settings have changed it returns true,
		// if the settings are unchanged, it returns false
    virtual bool saveTempSettings();
		// save all temporary key/value pairs to an additional file
		virtual void saveTempSettingsToAdditionalFile(const QString &fileName);

		// if the samba settings are stale, re-read smb.conf (update settings)
		virtual bool refresh();

		// save key value pair in a temporary location
		virtual void updateTempKey(const char *key, const char *value);
		// query current value of temporary key
		virtual const char * valueAtTempKey(const char *key);

		// query actual value of key from smb.conf file
		virtual const char * valueAtKey(const char *key);

		// updates smb.conf parameters relating to server settings 
		virtual void updateTempServerSettings(bool bPDC, bool bLocalMaster);

		// creates (if absent) special [netlogon] share for Win9X/NT PDC in smb.conf
		virtual bool updatePDCNetlogonShare();

		// if the service exists, restart the samba daemons
		virtual void restartSamba();
		// if the service doesn't exist, enable the samba service
		virtual void enableSamba();
		// if the service exists, remove the samba service
		virtual void disableSamba();

		// returns a boolean indicating whether the samba service script was found
		virtual bool sambaServiceFound();
		// returns a boolean indicating whether samba is currently activated
		virtual bool sambaServiceActivated();

		// returns true if both the nmbd and smbd daemons are currently running (listening)
		virtual bool sambaDaemonsRunning();

protected:
		// search for the samba service script and smb.conf, read initial values
		virtual void initialize();

		// search the possible locations to find smb.conf
    virtual void searchSmbConfLocation();
		// search the possible locations to find the samba service script
		virtual void searchSMBServiceLocation();

		// returns true if strHostName is listening for strProtocol packets on port nPort
		virtual bool isPortAcceptingConnections(int nPort, 
																						const char *strProtocol, 
																						const char *strHostName);
		
		// find and return the global section within the global samba section list
    virtual CSmbSection* findGlobalSection();
		// find and return the global section within the given samba section list
		virtual CSmbSection* findGlobalSectionInList(CSmbSectionList &list);

protected:
   CSmbSection*	m_psGlobalSambaSection;

   QString	m_strSmbConfLocation;
	 time_t m_SmbConfLastModified;
   BOOL		m_bSmbConfReadOnly;
   BOOL		m_bNetworkAvailable;

	 QString m_strSMB_SERVICE_NAME;
	 QString m_strRUN_LEVEL_PARENT_PATH;

	 bool m_bSMBServiceFound;

	 QDict<char> m_dictSMBSettings;
};


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class KRunLevelServicesBuilder is used to add entries and remove entries from an rc*.d directory

class KRunLevelServicesBuilder
{
public:
	KRunLevelServicesBuilder( LPCSTR _szRCParentPath  = "" )		{ setRunLevelParentPath(  _szRCParentPath );  }
	virtual ~KRunLevelServicesBuilder()					{}

	void setRunLevelParentPath( LPCSTR _szRCParentPath)	{ m_strRCParentPath = _szRCParentPath; }
	const QString& runLevelParentPath() const			{ return m_strRCParentPath; }

	BOOL serviceExists( LPCSTR _szServiceName ) const			{ return serviceExists( m_strRCParentPath, _szServiceName ); }
	
	void startService( LPCSTR _szServiceName ) const			{ startService( m_strRCParentPath, _szServiceName ); }
	void stopService( LPCSTR _szServiceName ) const			{ stopService( m_strRCParentPath, _szServiceName ); }
	void restartService( LPCSTR _szServiceName ) const			{ restartService( m_strRCParentPath, _szServiceName ); }

	BOOL serviceActivatedAt( LPCSTR _szServiceName, INT _nRunLevel ) const	{ return serviceActivatedAt( (LPCSTR)m_strRCParentPath, _szServiceName, _nRunLevel ); }
	// run level lists should end with a (-1)
	BOOL serviceActivatedAt( LPCSTR _szServiceName,  INT nFirstRunLevel,  INT nSecondRunLevel, /*rest of run levels*/... ) const;

	void addService( LPCSTR _szServiceName, INT _nRunLevel ) const	{ addService( (LPCSTR)m_strRCParentPath, _szServiceName, _nRunLevel ); }
	// run level lists should end with a (-1)
	void addService( LPCSTR _szServiceName,  INT nFirstRunLevel,  INT nSecondRunLevel, /*rest of run levels*/... ) const;

	void removeService( LPCSTR _szServiceName, INT _nRunLevel ) const	{ removeService( (LPCSTR)m_strRCParentPath, _szServiceName, _nRunLevel ); }
	// run level lists should end with a (-1)
	void removeService( LPCSTR _szServiceName,  INT nFirstRunLevel,  INT nSecondRunLevel, /*rest of run levels*/... ) const;
	
	static BOOL serviceExists( LPCSTR _szRCParentPath, LPCSTR _szServiceName );
	static void startService( LPCSTR _szRCParentPath, LPCSTR _szServiceName );
	static void stopService( LPCSTR _szRCParentPath, LPCSTR _szServiceName );
	static void restartService( LPCSTR _szRCParentPath, LPCSTR _szServiceName );

	static BOOL serviceActivatedAt( LPCSTR _szRCParentPath, LPCSTR _szServiceName, INT _nRunLevel );
	// run level lists should end with a (-1)
	static BOOL serviceActivatedAt( LPCSTR _szRCParentPath,  LPCSTR _szServiceName,  INT nFirstRunLevel,  INT nSecondRunLevel, /*rest of run levels*/... );

	static void addService( LPCSTR _szRCParentPath, LPCSTR _szServiceName, INT _nRunLevel );
	// run level lists should end with a (-1)
	static void addService( LPCSTR _szRCParentPath,  LPCSTR _szServiceName,  INT nFirstRunLevel, INT nSecondRunLevel, /*rest of run levels*/... );

	static void removeService( LPCSTR _szRCParentPath, LPCSTR _szServiceName, INT _nRunLevel );
	// run level lists should end with a (-1)
	static void removeService( LPCSTR _szRCParentPath,  LPCSTR _szServiceName,  INT nFirstRunLevel,  INT nSecondRunLevel, /*rest of run levels*/... );
// typedefinitions
protected:
	typedef enum { eStart=0, eStop, eRestart } eRUN_SCRIPT_TYPE;
// helper methods
protected:
	static void runServiceScript( LPCSTR _szRCParentPath, LPCSTR _szServiceName, eRUN_SCRIPT_TYPE _eRunType );

	// returns the integer that matches the asterix in the 2nd parameter in the specified path
	// on error returns -1, if not found, returns 0
	static int findFirstNumForService(const char *strPathName, const char *strServiceMatch);
	// returns an integer representing the default run-level for the local host
	static int findDefaultRunLevel(const char *inittabFileName);

	static INT commandExec ( LPCSTR cszCommand );

// attributes
protected:
	QString	m_strRCParentPath;
};
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif // __CSambaConfig__

