//  -*-C++-*-        (magic first line for emacs)
//wizardpage.h
//
// Author: Florin Epure
// Revisor 1: Emily Ezust
//
// Copyright (C) 1999 Corel Corp. All rights reserved
//
// Revision	Date		Description
//
// 1.00		22/06/99	Created
// 1.01                         Added more flexibility in button management
//
////////////////////////////////////////////////////////////////////////////////

#ifndef WIZARDPAGE_H
#define WIZARDPAGE_H
#include <qwidget.h>

class CCorelWizard;

//wizard page
class CWizardPage : public QWidget
{
  Q_OBJECT
public:
  CWizardPage( QWidget *parent=0, const char *name=0, WFlags f=0 );

  CWizardPage( int nPage, bool bFinalPage = false, int nNextPage = -1,
	       int nPrevPage = -1, const char* pszImage = 0,
	       const char* pszTitle = 0, bool bCheckPage = true,
		QWidget *parent=0, const char *name=0 );

  //set current wizard page
  void setPage( int nPage, bool bFinalPage = false, int nNextPage = -1,
		int nPrevPage = -1, const char* pszImage = 0,
		const char* pszTitle = 0, bool bCheckPage = true );
	
  //virtual override to allow for wizards with mulitple "final" pages
  //if you return true for isFinalPage, wizard will disable "next" button
  //even if this is not last page on the wizard
  virtual bool isFinalPage() { return m_bFinalPage; }
	
  //if you want to set "custom" page number for "next" or "previous" page
  //return TRUE and set correct page number in pointer variable
  //return FALSE will implement the default next/previous scheme
  virtual bool getNextPage( int* pnNextPage )
    { *pnNextPage = m_nNextPage; return m_nNextPage == -1 ? false : true ; }
  virtual bool getPrevPage( int* pnPrevPage )
    { *pnPrevPage = m_nPrevPage; return m_nPrevPage == -1 ? false : true; }

  //if you have an image that you want to left of wizard
  //return pointer to pixmap file name
  virtual bool getImage(const char** ppImage)
  {
    *ppImage = m_pszImage;
    return m_pszImage == 0 ? false : (m_pszImage[0] == '\0' ? false : true);
  }

  //if you have a title that you want to upper side of wizard
  //return pointer to title - font size is set by wizard
  virtual bool getTitle( const char** ppTitle )
  {
    *ppTitle = m_pszTitle;
    return m_pszTitle == 0 ? false : (m_pszTitle[0] == '\0' ? false : true);
  }

  int getPageNumber() { return m_nPage; }

  //if you want to check contents of current page over-ride this function.
  //It used to be called with a bool telling you whether eButtonNext had been
  //clicked, but now it is called with nButton.

  virtual bool checkPage(int nButton) { return m_bCheckPage; }

  //prepare contents of current page before displaying
  virtual void preparePage() { }
  virtual void prepareButtons() {}

public slots:
  void pressedButton( int nButton, int nPage );

  //enable/grey-out a button display in wizard
  void enableButton( int nButton, bool bEnable );

protected:
  void paintEvent( QPaintEvent* event );
protected:
  bool m_bFinalPage;
  int m_nNextPage;
  int m_nPrevPage;
  QString m_pszImage;
  QString m_pszTitle;
  int m_nPage;   // which page the wizardpage was installed as
  CCorelWizard *m_pWizard;  // pointer to the wizard
  bool m_bCheckPage;
};

//prototypes for shared library wizards functions

typedef CWizardPage* (*PGetClassPtr)(int nPage, bool bFinalPage = false,
				     int nNextPage = -1, int nPrevPage = -1,
				     const char* pImage = 0,
				     const char* pTitle = 0,
				     bool bCheckPage = true,
				     QWidget *parent=0, const char *name=0 );

//create a structure with function pointers on these functions

typedef struct _SWIZARDPAGE
{
  void* pHandle;
  PGetClassPtr pGetClassPtr;
} SWIZARDPAGE, *PSWIZARDPAGE;

#define IDS_WIZARD_FILE		"config.ini"
#define IDS_WIZARD_INIT		"WizardInit"
#define IDS_WIZARD_COUNT	"PageCount"
#define IDS_WIZARD_PAGE		"WizardPage%d"
#define IDS_WIZARD_LIBRARY	"Library"
#define IDS_WIZARD_NEXT		"NextPage"
#define IDS_WIZARD_PREV		"PrevPage"
#define IDS_WIZARD_IMAGE	"Image"
#define IDS_WIZARD_TITLE	"Title"
#define IDS_WIZARD_ATTRIBUTE	"PageAttribute"
#define IDS_WIZARD_ACTIVE	"ActivePage"
#define IDS_WIZARD_FINAL	"FinalPage"
#define IDS_WIZARD_CHECK	"CheckPage"

/*

  Here is  an example generated with qtarchitect. It invokes constructor
of a custom class with three parameters. Modifications in QWizardPage class
accomodate this specific invocation. So in conclusion you can define a wizard
page in two ways: either by specifying all parameters in constructor, or by
calling an empty constructor and calling a "set" function.

//user.h
class user : public userData
{
public:
    user
    (
        QWidget* parent = NULL,
        const char* name = NULL
    );

    virtual ~user();

};

//userData.h
#include "wizardpage.h"

class userData : public CWizardPage
{
public:
    userData
    (
        QWidget* parent = NULL,
        const char* name = NULL
    );

    virtual ~userData();

};

//userData.cpp

#define Inherited QWizardPage

userData::userData
(
	QWidget* parent,
	const char* name
)
	:
	Inherited( parent, name, 0 )
{
}

	Following code is an example of using the wizard class with user class:

#include ...

//globals

QCorelWizard* pw=0;
QWizardPage* pg=0;

bool init()
{
	bool bRet = true;
	
	pw = new CSetupWizard(IDS_PREV,IDS_NEXT,IDS_DONE,IDS_HELP,IDS_ABORT);

	int i = 0;

	pg = new user(pw);
	pg->setPage(i++, false, -1, -1, IDS_PIXMAP1, IDS_TITLE1, true);
	//this is default page
	pw->addPage(pg,eButtonPrev | eButtonNext | eButtonHelp | eButtonCancel,false );

	return bRet;
}

	If you used the base class:

bool init()
{
	pg = new CWizardPage( i++, false, -1, -1, IDS_PIXMAP1, IDS_TITLE1, true, pw );
	//this is default page
	pw->addPage(pg,eButtonPrev | eButtonNext | eButtonHelp | eButtonCancel,true );
	return true;
}

int main( int argc, char **argv )
{
	QApplication a( argc, argv );

	if (!init( ) )
	{
		printf("error initialzing - exit\n");
		exit(1);
	}

	a.setMainWidget( pw );
	//you MUST call these two functions for correct function of wizard
	pw->setSize( 450, 300 );
	pw->setPos( 100,100 );
	pw->show();
	return a.exec();
}

*/

#endif//WIZARDPAGE_H
